package Align3TP;

/**
 * Represents a vector.
 *
 * @author J. Anthony Parker, MD PhD <J.A.Parker@IEEE.org>
 * @version 7January2002
 *
 * @see Align3TP.Affine
 * @see Align3TP.Matrix
 */

class Vector {
	private double[] v;

	Vector(int length) {
		v = new double[length];
		for(int i=0; i<length; i++)
			v[i] = 0.0;
		return;
	}

	Vector(Vector in) {
		v = new double[in.length()];
		for(int i=0; i<v.length; i++)
			v[i] = in.v[i];
		return;
	}

	Vector(double[] v) {
		this.v = new double[v.length];
		for(int i=0; i<v.length; i++)
			this.v[i] = v[i];
		return;
	}

	int length() {
		return v.length;
	}

	double[] value() {
		return v;
	}

	// out = v+in
	// out = null if len of v not equal to len of in
	Vector add(Vector in) {
		if(v.length!=in.length()) return null;
		Vector out = new Vector(v.length);
		for(int i=0; i<v.length; i++)
				out.v[i] = this.v[i]+in.v[i];
		return out;
	}

	// out = v-in
	// out = null if len of v not equal to len of in
	Vector sub(Vector in) {
		if(v.length!=in.length()) return null;
		Vector out = new Vector(v.length);
		for(int i=0; i<v.length; i++)
				out.v[i] = this.v[i]-in.v[i];
		return out;
	}

	Vector minus() {
		double[] out = new double[v.length];
		for(int i=0; i<v.length; i++)
			out[i] = -v[i];
		return new Vector(out);
	}

	double multiply(Vector in) {
		if(v.length!=in.length()) return Double.NaN;
		double x = 0.0;
		for(int i=0; i<v.length; i++)
			x += this.v[i]*in.v[i];
		return x;
	}

	double vectorLength() {
		double lenSq = 0.0;
		for(int i=0; i<v.length; i++)
			lenSq += v[i]*v[i];
		return Math.sqrt(lenSq);
	}

	/* unused code
	double innerAngle(Vector in) {
		return Math.acos(this.multiply(in)/
						(this.vectorLength()*in.vectorLength()));
	}
	*/

	public String toString() {
		if(v.length==0) return "Zero dimension.\n";
		String s = "";
		Double d  = new Double(0.0);
		Integer n = new Integer(0);
		for(int i=0; i<v.length; i++) {
				s = s+"v("+n.toString(i)+") = "+
					d.toString(Math.round(1000.0*v[i])/1000.0)+", ";
		}
		s = s.substring(0,s.length()-2)+"\n";
		return s;
	}
} // end of the Vector class

